@@ -38,4 +38,14 @@ struct fake_file_handle {
 	void *private_data;
 };
 
+enum FAKE_FILE_TYPE is_fake_file_path(const char *path);
+enum FAKE_FILE_TYPE is_fake_file(const struct smb_filename *smb_fname);
+NTSTATUS open_fake_file(struct smb_request *req, connection_struct *conn,
+				uint16_t current_vuid,
+				enum FAKE_FILE_TYPE fake_file_type,
+				const struct smb_filename *smb_fname,
+				uint32 access_mask,
+				files_struct **result);
+NTSTATUS close_fake_file(struct smb_request *req, files_struct *fsp);
+
 #endif /* _FAKE_FILE_H */
