@@ -46,8 +46,8 @@ static void remove_pending_lock(struct smbd_smb2_lock_state *state,
 static struct tevent_req *smbd_smb2_lock_send(TALLOC_CTX *mem_ctx,
 						 struct tevent_context *ev,
 						 struct smbd_smb2_request *smb2req,
+						 struct files_struct *in_fsp,
 						 uint32_t in_smbpid,
-						 uint64_t in_file_id_volatile,
 						 uint16_t in_lock_count,
 						 struct smbd_smb2_lock_element *in_locks);
 static NTSTATUS smbd_smb2_lock_recv(struct tevent_req *req);
@@ -58,29 +58,24 @@ NTSTATUS smbd_smb2_request_process_lock(struct smbd_smb2_request *req)
 	const uint8_t *inhdr;
 	const uint8_t *inbody;
 	const int i = req->current_idx;
-	size_t expected_body_size = 0x30;
-	size_t body_size;
 	uint32_t in_smbpid;
 	uint16_t in_lock_count;
 	uint64_t in_file_id_persistent;
 	uint64_t in_file_id_volatile;
+	struct files_struct *in_fsp;
 	struct smbd_smb2_lock_element *in_locks;
 	struct tevent_req *subreq;
 	const uint8_t *lock_buffer;
 	uint16_t l;
+	NTSTATUS status;
 
+	status = smbd_smb2_request_verify_sizes(req, 0x30);
+	if (!NT_STATUS_IS_OK(status)) {
+		return smbd_smb2_request_error(req, status);
+	}
 	inhdr = (const uint8_t *)req->in.vector[i+0].iov_base;
-	if (req->in.vector[i+1].iov_len != (expected_body_size & 0xFFFFFFFE)) {
-		return smbd_smb2_request_error(req, NT_STATUS_INVALID_PARAMETER);
-	}
-
 	inbody = (const uint8_t *)req->in.vector[i+1].iov_base;
 
-	body_size = SVAL(inbody, 0x00);
-	if (body_size != expected_body_size) {
-		return smbd_smb2_request_error(req, NT_STATUS_INVALID_PARAMETER);
-	}
-
 	in_smbpid			= IVAL(inhdr, SMB2_HDR_PID);
 
 	in_lock_count			= CVAL(inbody, 0x02);
@@ -96,12 +91,6 @@ NTSTATUS smbd_smb2_request_process_lock(struct smbd_smb2_request *req)
 		return smbd_smb2_request_error(req, NT_STATUS_INVALID_PARAMETER);
 	}
 
-	if (req->compat_chain_fsp) {
-		/* skip check */
-	} else if (in_file_id_persistent != in_file_id_volatile) {
-		return smbd_smb2_request_error(req, NT_STATUS_FILE_CLOSED);
-	}
-
 	in_locks = talloc_array(req, struct smbd_smb2_lock_element,
 				in_lock_count);
 	if (in_locks == NULL) {
@@ -127,11 +116,14 @@ NTSTATUS smbd_smb2_request_process_lock(struct smbd_smb2_request *req)
 		lock_buffer += 0x18;
 	}
 
-	subreq = smbd_smb2_lock_send(req,
-				     req->sconn->smb2.event_ctx,
-				     req,
+	in_fsp = file_fsp_smb2(req, in_file_id_persistent, in_file_id_volatile);
+	if (in_fsp == NULL) {
+		return smbd_smb2_request_error(req, NT_STATUS_FILE_CLOSED);
+	}
+
+	subreq = smbd_smb2_lock_send(req, req->sconn->smb2.event_ctx,
+				     req, in_fsp,
 				     in_smbpid,
-				     in_file_id_volatile,
 				     in_lock_count,
 				     in_locks);
 	if (subreq == NULL) {
@@ -213,16 +205,14 @@ static void smbd_smb2_request_lock_done(struct tevent_req *subreq)
 static struct tevent_req *smbd_smb2_lock_send(TALLOC_CTX *mem_ctx,
 						 struct tevent_context *ev,
 						 struct smbd_smb2_request *smb2req,
+						 struct files_struct *fsp,
 						 uint32_t in_smbpid,
-						 uint64_t in_file_id_volatile,
 						 uint16_t in_lock_count,
 						 struct smbd_smb2_lock_element *in_locks)
 {
 	struct tevent_req *req;
 	struct smbd_smb2_lock_state *state;
 	struct smb_request *smb1req;
-	connection_struct *conn = smb2req->tcon->compat_conn;
-	files_struct *fsp;
 	int32_t timeout = -1;
 	bool isunlock = false;
 	uint16_t i;
@@ -244,22 +234,8 @@ static struct tevent_req *smbd_smb2_lock_send(TALLOC_CTX *mem_ctx,
 	}
 	state->smb1req = smb1req;
 
-	DEBUG(10,("smbd_smb2_lock_send: file_id[0x%016llX]\n",
-		  (unsigned long long)in_file_id_volatile));
-
-	fsp = file_fsp(smb1req, (uint16_t)in_file_id_volatile);
-	if (fsp == NULL) {
-		tevent_req_nterror(req, NT_STATUS_FILE_CLOSED);
-		return tevent_req_post(req, ev);
-	}
-	if (conn != fsp->conn) {
-		tevent_req_nterror(req, NT_STATUS_FILE_CLOSED);
-		return tevent_req_post(req, ev);
-	}
-	if (smb2req->session->vuid != fsp->vuid) {
-		tevent_req_nterror(req, NT_STATUS_FILE_CLOSED);
-		return tevent_req_post(req, ev);
-	}
+	DEBUG(10,("smbd_smb2_lock_send: %s - fnum[%d]\n",
+		  fsp_str_dbg(fsp), fsp->fnum));
 
 	locks = talloc_array(state, struct smbd_lock_element, in_lock_count);
 	if (locks == NULL) {
@@ -341,7 +317,7 @@ static struct tevent_req *smbd_smb2_lock_send(TALLOC_CTX *mem_ctx,
 			return tevent_req_post(req, ev);
 		}
 
-		locks[i].smblctx = in_file_id_volatile;
+		locks[i].smblctx = fsp->fnum;
 		locks[i].offset = in_locks[i].offset;
 		locks[i].count  = in_locks[i].length;
 
@@ -872,10 +848,8 @@ void cancel_pending_lock_requests_by_fid_smb2(files_struct *fsp,
 		struct smbd_smb2_lock_state *state = NULL;
 		files_struct *fsp_curr = NULL;
 		int i = smb2req->current_idx;
-		uint64_t in_file_id_volatile;
 		struct blocking_lock_record *blr = NULL;
 		const uint8_t *inhdr;
-		const uint8_t *inbody;
 
 		nextreq = smb2req->next;
 
@@ -894,9 +868,6 @@ void cancel_pending_lock_requests_by_fid_smb2(files_struct *fsp,
 			continue;
 		}
 
-		inbody = (const uint8_t *)smb2req->in.vector[i+1].iov_base;
-		in_file_id_volatile = BVAL(inbody, 0x10);
-
 		state = tevent_req_data(smb2req->subreq,
 				struct smbd_smb2_lock_state);
 		if (!state) {
@@ -904,7 +875,7 @@ void cancel_pending_lock_requests_by_fid_smb2(files_struct *fsp,
 			continue;
 		}
 
-		fsp_curr = file_fsp(state->smb1req, (uint16_t)in_file_id_volatile);
+		fsp_curr = smb2req->compat_chain_fsp;
 		if (fsp_curr == NULL) {
 			/* Strange - is this even possible ? */
 			continue;
